@@ -10,8 +10,10 @@ import (
 	"github.com/lucas-clemente/quic-go/congestion"
 	"github.com/lunixbochs/struc"
 	"github.com/tobyxdd/hysteria/pkg/obfs"
-	transport2 "github.com/tobyxdd/hysteria/pkg/transport"
+	"github.com/tobyxdd/hysteria/pkg/pmtud_fix"
+	"github.com/tobyxdd/hysteria/pkg/transport"
 	"github.com/tobyxdd/hysteria/pkg/utils"
+	"math/rand"
 	"net"
 	"strconv"
 	"sync"
@@ -25,7 +27,7 @@ var (
 type CongestionFactory func(refBPS uint64) congestion.CongestionControl
 
 type Client struct {
-	transport         transport2.Transport
+	transport         *transport.ClientTransport
 	serverAddr        string
 	protocol          string
 	sendBPS, recvBPS  uint64
@@ -36,17 +38,19 @@ type Client struct {
 	tlsConfig  *tls.Config
 	quicConfig *quic.Config
 
-	quicSession    quic.Session
+	quicSession    quic.Connection
 	reconnectMutex sync.Mutex
 	closed         bool
 
 	udpSessionMutex sync.RWMutex
 	udpSessionMap   map[uint32]chan *udpMessage
+	udpDefragger    defragger
 }
 
 func NewClient(serverAddr string, protocol string, auth []byte, tlsConfig *tls.Config, quicConfig *quic.Config,
-	transport transport2.Transport, sendBPS uint64, recvBPS uint64, congestionFactory CongestionFactory,
+	transport *transport.ClientTransport, sendBPS uint64, recvBPS uint64, congestionFactory CongestionFactory,
 	obfuscator obfs.Obfuscator) (*Client, error) {
+	quicConfig.DisablePathMTUDiscovery = quicConfig.DisablePathMTUDiscovery || pmtud_fix.DisablePathMTUDiscovery
 	c := &Client{
 		transport:         transport,
 		serverAddr:        serverAddr,
@@ -66,19 +70,10 @@ func NewClient(serverAddr string, protocol string, auth []byte, tlsConfig *tls.C
 }
 
 func (c *Client) connectToServer() error {
-	serverUDPAddr, err := c.transport.QUICResolveUDPAddr(c.serverAddr)
+	qs, err := c.transport.QUICDial(c.protocol, c.serverAddr, c.tlsConfig, c.quicConfig, c.obfuscator)
 	if err != nil {
 		return err
 	}
-	pktConn, err := c.transport.QUICPacketConn(c.protocol, false, "", c.serverAddr, c.obfuscator)
-	if err != nil {
-		return err
-	}
-	qs, err := quic.Dial(pktConn, serverUDPAddr, c.serverAddr, c.tlsConfig, c.quicConfig)
-	if err != nil {
-		_ = pktConn.Close()
-		return err
-	}
 	// Control stream
 	ctx, ctxCancel := context.WithTimeout(context.Background(), protocolTimeout)
 	stream, err := qs.OpenStreamSync(ctx)
@@ -103,7 +98,7 @@ func (c *Client) connectToServer() error {
 	return nil
 }
 
-func (c *Client) handleControlStream(qs quic.Session, stream quic.Stream) (bool, string, error) {
+func (c *Client) handleControlStream(qs quic.Connection, stream quic.Stream) (bool, string, error) {
 	// Send protocol version
 	_, err := stream.Write([]byte{protocolVersion})
 	if err != nil {
@@ -133,7 +128,7 @@ func (c *Client) handleControlStream(qs quic.Session, stream quic.Stream) (bool,
 	return sh.OK, sh.Message, nil
 }
 
-func (c *Client) handleMessage(qs quic.Session) {
+func (c *Client) handleMessage(qs quic.Connection) {
 	for {
 		msg, err := qs.ReceiveMessage()
 		if err != nil {
@@ -144,11 +139,15 @@ func (c *Client) handleMessage(qs quic.Session) {
 		if err != nil {
 			continue
 		}
+		dfMsg := c.udpDefragger.Feed(udpMsg)
+		if dfMsg == nil {
+			continue
+		}
 		c.udpSessionMutex.RLock()
-		ch, ok := c.udpSessionMap[udpMsg.SessionID]
+		ch, ok := c.udpSessionMap[dfMsg.SessionID]
 		if ok {
 			select {
-			case ch <- &udpMsg:
+			case ch <- dfMsg:
 				// OK
 			default:
 				// Silently drop the message when the channel is full
@@ -158,7 +157,7 @@ func (c *Client) handleMessage(qs quic.Session) {
 	}
 }
 
-func (c *Client) openStreamWithReconnect() (quic.Session, quic.Stream, error) {
+func (c *Client) openStreamWithReconnect() (quic.Connection, quic.Stream, error) {
 	c.reconnectMutex.Lock()
 	defer c.reconnectMutex.Unlock()
 	if c.closed {
@@ -327,7 +326,7 @@ type UDPConn interface {
 }
 
 type quicPktConn struct {
-	Session      quic.Session
+	Session      quic.Connection
 	Stream       quic.Stream
 	CloseFunc    func()
 	UDPSessionID uint32
@@ -360,14 +359,38 @@ func (c *quicPktConn) WriteTo(p []byte, addr string) error {
 	if err != nil {
 		return err
 	}
-	var msgBuf bytes.Buffer
-	_ = struc.Pack(&msgBuf, &udpMessage{
+	msg := udpMessage{
 		SessionID: c.UDPSessionID,
 		Host:      host,
 		Port:      port,
+		FragCount: 1,
 		Data:      p,
-	})
-	return c.Session.SendMessage(msgBuf.Bytes())
+	}
+	// try no frag first
+	var msgBuf bytes.Buffer
+	_ = struc.Pack(&msgBuf, &msg)
+	err = c.Session.SendMessage(msgBuf.Bytes())
+	if err != nil {
+		if errSize, ok := err.(quic.ErrMessageToLarge); ok {
+			// need to frag
+			msg.MsgID = uint16(rand.Intn(0xFFFF)) + 1 // msgID must be > 0 when fragCount > 1
+			fragMsgs := fragUDPMessage(msg, int(errSize))
+			for _, fragMsg := range fragMsgs {
+				msgBuf.Reset()
+				_ = struc.Pack(&msgBuf, &fragMsg)
+				err = c.Session.SendMessage(msgBuf.Bytes())
+				if err != nil {
+					return err
+				}
+			}
+			return nil
+		} else {
+			// some other error
+			return err
+		}
+	} else {
+		return nil
+	}
 }
 
 func (c *quicPktConn) Close() error {
